use CGI;
print CGI->header({charset => 'utf-8'});
$PerlIO::via::trap::PASS = 1;
open my $out, '>>/tmp/trapped.log' or die $!;
print $out $ENV{REDIRECT_ERROR_NOTES};
close $out;
print << '.';
<H2>給錯</H2>
<PRE><FONT SIZE=+1>
常喜歡你這樣子
坐著，散起頭髮，彈一些些的杜步西
在折斷了的牛蒡上
在河裏的雲上
天藍著漢代的藍
基督溫柔古昔的溫柔
在水磨的遠處在雀聲下
在靠近五月的時候

（讓他們喊他們的酢醬草萬歲）

整整的一生是多麼地、多麼地長啊
縱有某種詛咒久久停在
豎笛和低音簫們那裏
而從朝至暮念著他、惦著他是多麼的美麗

想著，生活著，偶而也微笑著
既不快活也不不快活
有一些甚麼在你頭上飛翔
或許
從沒一些甚麼

美麗的禾束時時配置在田地上
他總吻在他喜歡吻的地方
可曾瞧見陣雨打濕了樹葉與草麼
要作草與葉
或是作陣雨
隨你的意

（讓他們喊他們的酢醬草萬歲）

下午總愛吟那闋「聲聲慢」
修著指甲，坐著飲茶
整整的一生是多麼長啊
在過去歲月的額上
在疲倦的語字間

整整一生是多麼長啊
在一支歌的擊打下
在悔恨裏

任誰也不說那樣的話
那樣的話，那樣的呢
遂心亂了，遂失落了
遠遠地，遠遠遠遠地
</FONT></PRE>
<HR>
此問題已通知系統管理人員。抱歉。<br />
<a href="/ournet.pl">回首頁</a>
.
$PerlIO::via::trap::PASS = 0;
